@@ -194,4 +194,47 @@
 
 #define HASH_WALK_FILTER_END } while (0)
 
+
+static inline void
+mem_hash_init(u64 *h)
+{
+  *h = 0x001047d54778bcafULL;
+}
+
+static inline void
+mem_hash_mix(u64 *h, const void *p, uint s)
+{
+  const u64 multiplier = 0xb38bc09a61202731ULL;
+  const char *pp = p;
+  uint i;
+
+  for (i=0; i<s/4; i++)
+    *h = *h * multiplier + ((const u32 *)pp)[i];
+
+  for (i=s & ~0x3; i<s; i++)
+    *h = *h * multiplier + pp[i];
+}
+
+static inline uint
+mem_hash_value(u64 *h)
+{
+  return ((*h >> 32) ^ (*h & 0xffffffff));
+}
+
+static inline uint
+mem_hash(const void *p, uint s)
+{
+  static u64 h;
+  mem_hash_init(&h);
+  mem_hash_mix(&h, p, s);
+  return mem_hash_value(&h);
+}
+
+static inline uint
+ptr_hash(void *ptr)
+{
+  uintptr_t p = (uintptr_t) ptr;
+  return p ^ (p << 8) ^ (p >> 16);
+}
+
 #endif
