@@ -13,6 +13,8 @@
 #include <string.h>
 #include <strings.h>
 
+#include "lib/resource.h"
+
 int bsprintf(char *str, const char *fmt, ...);
 int bvsprintf(char *str, const char *fmt, va_list args);
 int bsnprintf(char *str, int size, const char *fmt, ...);
@@ -22,6 +24,9 @@ int buffer_vprint(buffer *buf, const char *fmt, va_list args);
 int buffer_print(buffer *buf, const char *fmt, ...);
 void buffer_puts(buffer *buf, const char *str);
 
+u64 bstrtoul10(const char *str, char **end);
+u64 bstrtoul16(const char *str, char **end);
+
 int patmatch(const byte *pat, const byte *str);
 
 static inline char *xbasename(const char *str)
@@ -39,6 +44,15 @@ xstrdup(const char *c)
   return z;
 }
 
+static inline char *
+lp_strdup(linpool *lp, const char *c)
+{
+  size_t l = strlen(c) + 1;
+  char *z = lp_allocu(lp, l);
+  memcpy(z, c, l);
+  return z;
+}
+
 static inline void
 memset32(void *D, u32 val, uint n)
 {
@@ -49,6 +63,15 @@ memset32(void *D, u32 val, uint n)
     dst[i] = val;
 }
 
+static inline int
+bstrcmp(const char *s1, const char *s2)
+{
+  if (s1 && s2)
+    return strcmp(s1, s2);
+  else
+    return !s2 - !s1;
+}
+
 #define ROUTER_ID_64_LENGTH 23
 
 #endif
