@@ -60,16 +60,18 @@ static inline struct adata *as_path_prepend(struct linpool *pool, const struct a
 #define PM_ASTERISK	2
 #define PM_ASN_EXPR	3
 #define PM_ASN_RANGE	4
+#define PM_ASN_SET	5
 
 struct f_path_mask_item {
   union {
     u32 asn; /* PM_ASN */
-    struct f_line *expr; /* PM_ASN_EXPR */
+    const struct f_line *expr; /* PM_ASN_EXPR */
+    const struct f_tree *set; /* PM_ASN_SET */
     struct { /* PM_ASN_RANGE */
       u32 from;
       u32 to;
     };
-  }; 
+  };
   int kind;
 };
 
