@@ -61,6 +61,7 @@
 const adata null_adata;		/* adata of length 0 */
 
 const char * const rta_src_names[RTS_MAX] = {
+  [RTS_DUMMY]		= "",
   [RTS_STATIC]		= "static",
   [RTS_INHERIT]		= "inherit",
   [RTS_DEVICE]		= "device",
@@ -134,7 +135,7 @@ rt_get_source(struct proto *p, u32 id)
   if (src)
     return src;
 
-  src = sl_allocz(rte_src_slab);
+  src = sl_alloc(rte_src_slab);
   src->proto = p;
   src->private_id = id;
   src->global_id = idm_alloc(&src_ids);
@@ -154,7 +155,7 @@ rt_prune_sources(void)
     {
       HASH_DO_REMOVE(src_hash, RSH, sp);
       idm_free(&src_ids, src->global_id);
-      sl_free(src);
+      sl_free(rte_src_slab, src);
     }
   }
   HASH_WALK_FILTER_END;
@@ -201,7 +202,7 @@ nexthop__same(struct nexthop *x, struct nexthop *y)
 }
 
 static int
-nexthop_compare_node(const struct nexthop *x, const struct nexthop *y)
+nexthop_compare_node(const struct nexthop *x, const  struct nexthop *y)
 {
   int r;
 
@@ -277,22 +278,18 @@ nexthop_merge(struct nexthop *x, struct nexthop *y, int rx, int ry, int max, lin
   while ((x || y) && max--)
   {
     int cmp = nexthop_compare_node(x, y);
-
     if (cmp < 0)
     {
-      ASSUME(x);
       *n = rx ? x : nexthop_copy_node(x, lp);
       x = x->next;
     }
     else if (cmp > 0)
     {
-      ASSUME(y);
       *n = ry ? y : nexthop_copy_node(y, lp);
       y = y->next;
     }
     else
     {
-      ASSUME(x && y);
       *n = rx ? x : (ry ? y : nexthop_copy_node(x, lp));
       x = x->next;
       y = y->next;
@@ -365,7 +362,7 @@ nexthop_copy(struct nexthop *o)
 
   for (; o; o = o->next)
     {
-      struct nexthop *n = sl_allocz(nexthop_slab(o));
+      struct nexthop *n = sl_alloc(nexthop_slab(o));
       n->gw = o->gw;
       n->iface = o->iface;
       n->next = NULL;
@@ -391,7 +388,7 @@ nexthop_free(struct nexthop *o)
   while (o)
     {
       n = o->next;
-      sl_free(o);
+      sl_free(nexthop_slab(o), o);
       o = n;
     }
 }
@@ -448,7 +445,8 @@ ea_find(ea_list *e, unsigned id)
 {
   eattr *a = ea__find(e, id & EA_CODE_MASK);
 
-  if (a && a->undef && !(id & EA_ALLOW_UNDEF))
+  if (a && (a->type & EAF_TYPE_MASK) == EAF_TYPE_UNDEF &&
+      !(id & EA_ALLOW_UNDEF))
     return NULL;
   return a;
 }
@@ -515,7 +513,7 @@ ea_walk(struct ea_walk_state *s, uint id, uint max)
 
 	BIT32_SET(s->visited, n);
 
-	if (a->undef)
+	if ((a->type & EAF_TYPE_MASK) == EAF_TYPE_UNDEF)
 	  continue;
 
 	s->eattrs = e;
@@ -539,8 +537,8 @@ ea_walk(struct ea_walk_state *s, uint id, uint max)
  * by calling ea_find() to find the attribute, extracting its value or returning
  * a provided default if no such attribute is present.
  */
-uintptr_t
-ea_get_int(ea_list *e, unsigned id, uintptr_t def)
+int
+ea_get_int(ea_list *e, unsigned id, int def)
 {
   eattr *a = ea_find(e, id);
   if (!a)
@@ -615,17 +613,14 @@ ea_do_prune(ea_list *e)
 
       /* Now s0 is the most recent version, s[-1] the oldest one */
       /* Drop undefs */
-      if (s0->undef)
+      if ((s0->type & EAF_TYPE_MASK) == EAF_TYPE_UNDEF)
 	continue;
 
       /* Copy the newest version to destination */
       *d = *s0;
 
       /* Preserve info whether it originated locally */
-      d->originated = s[-1].originated;
-
-      /* Not fresh any more, we prefer surstroemming */
-      d->fresh = 0;
+      d->type = (d->type & ~(EAF_ORIGINATED|EAF_FRESH)) | (s[-1].type & EAF_ORIGINATED);
 
       /* Next destination */
       d++;
@@ -739,9 +734,6 @@ ea_same(ea_list *x, ea_list *y)
       if (a->id != b->id ||
 	  a->flags != b->flags ||
 	  a->type != b->type ||
-	  a->originated != b->originated ||
-	  a->fresh != b->fresh ||
-	  a->undef != b->undef ||
 	  ((a->type & EAF_EMBEDDED) ? a->u.data != b->u.data : !adata_same(a->u.ptr, b->u.ptr)))
 	return 0;
     }
@@ -752,22 +744,14 @@ static inline ea_list *
 ea_list_copy(ea_list *o)
 {
   ea_list *n;
-  unsigned i, adpos, elen;
+  unsigned i, len;
 
   if (!o)
     return NULL;
   ASSERT(!o->next);
-  elen = adpos = sizeof(ea_list) + sizeof(eattr) * o->count;
-
-  for(i=0; i<o->count; i++)
-    {
-      eattr *a = &o->attrs[i];
-      if (!(a->type & EAF_EMBEDDED))
-	elen += sizeof(struct adata) + a->u.ptr->length;
-    }
-
-  n = mb_alloc(rta_pool, elen);
-  memcpy(n, o, adpos);
+  len = sizeof(ea_list) + sizeof(eattr) * o->count;
+  n = mb_alloc(rta_pool, len);
+  memcpy(n, o, len);
   n->flags |= EALF_CACHED;
   for(i=0; i<o->count; i++)
     {
@@ -775,31 +759,34 @@ ea_list_copy(ea_list *o)
       if (!(a->type & EAF_EMBEDDED))
 	{
 	  unsigned size = sizeof(struct adata) + a->u.ptr->length;
-	  ASSERT_DIE(adpos + size <= elen);
-
-	  struct adata *d = ((void *) n) + adpos;
+	  struct adata *d = mb_alloc(rta_pool, size);
 	  memcpy(d, a->u.ptr, size);
 	  a->u.ptr = d;
-
-	  adpos += size;
 	}
     }
-  ASSERT_DIE(adpos == elen);
   return n;
 }
 
 static inline void
 ea_free(ea_list *o)
 {
+  int i;
+
   if (o)
     {
       ASSERT(!o->next);
+      for(i=0; i<o->count; i++)
+	{
+	  eattr *a = &o->attrs[i];
+	  if (!(a->type & EAF_EMBEDDED))
+	    mb_free((void *) a->u.ptr);
+	}
       mb_free(o);
     }
 }
 
 static int
-get_generic_attr(const eattr *a, byte **buf, int buflen UNUSED)
+get_generic_attr(eattr *a, byte **buf, int buflen UNUSED)
 {
   if (a->id == EA_GEN_IGP_METRIC)
     {
@@ -811,7 +798,7 @@ get_generic_attr(const eattr *a, byte **buf, int buflen UNUSED)
 }
 
 void
-ea_format_bitfield(const struct eattr *a, byte *buf, int bufsize, const char **names, int min, int max)
+ea_format_bitfield(struct eattr *a, byte *buf, int bufsize, const char **names, int min, int max)
 {
   byte *bound = buf + bufsize - 32;
   u32 data = a->u.data;
@@ -907,7 +894,7 @@ ea_show_lc_set(struct cli *c, const struct adata *ad, byte *pos, byte *buf, byte
  * get_attr() hook, it's consulted first.
  */
 void
-ea_show(struct cli *c, const eattr *e)
+ea_show(struct cli *c, eattr *e)
 {
   struct protocol *p;
   int status = GA_UNKNOWN;
@@ -944,10 +931,6 @@ ea_show(struct cli *c, const eattr *e)
     {
       *pos++ = ':';
       *pos++ = ' ';
-
-      if (e->undef)
-	bsprintf(pos, "undefined");
-      else
       switch (e->type & EAF_TYPE_MASK)
 	{
 	case EAF_TYPE_INT:
@@ -977,6 +960,7 @@ ea_show(struct cli *c, const eattr *e)
 	case EAF_TYPE_LC_SET:
 	  ea_show_lc_set(c, ad, pos, buf, end);
 	  return;
+	case EAF_TYPE_UNDEF:
 	default:
 	  bsprintf(pos, "<type %02x>", e->type);
 	}
@@ -1012,7 +996,7 @@ ea_dump(ea_list *e)
 	  eattr *a = &e->attrs[i];
 	  debug(" %02x:%02x.%02x", EA_PROTO(a->id), EA_ID(a->id), a->flags);
 	  debug("=%c", "?iO?I?P???S?????" [a->type & EAF_TYPE_MASK]);
-	  if (a->originated)
+	  if (a->type & EAF_ORIGINATED)
 	    debug("o");
 	  if (a->type & EAF_EMBEDDED)
 	    debug(":%08x", a->u.data);
@@ -1111,14 +1095,13 @@ rta_hash(rta *a)
   u64 h;
   mem_hash_init(&h);
 #define MIX(f) mem_hash_mix(&h, &(a->f), sizeof(a->f));
-#define BMIX(f) mem_hash_mix_num(&h, a->f);
+  MIX(src);
   MIX(hostentry);
   MIX(from);
   MIX(igp_metric);
-  BMIX(source);
-  BMIX(scope);
-  BMIX(dest);
-  MIX(pref);
+  MIX(source);
+  MIX(scope);
+  MIX(dest);
 #undef MIX
 
   return mem_hash_value(&h) ^ nexthop_hash(&(a->nh)) ^ ea_hash(a->eattrs);
@@ -1127,7 +1110,8 @@ rta_hash(rta *a)
 static inline int
 rta_same(rta *x, rta *y)
 {
-  return (x->source == y->source &&
+  return (x->src == y->src &&
+	  x->source == y->source &&
 	  x->scope == y->scope &&
 	  x->dest == y->dest &&
 	  x->igp_metric == y->igp_metric &&
@@ -1205,7 +1189,7 @@ rta_lookup(rta *o)
   rta *r;
   uint h;
 
-  ASSERT(!o->cached);
+  ASSERT(!(o->aflags & RTAF_CACHED));
   if (o->eattrs)
     ea_normalize(o->eattrs);
 
@@ -1216,7 +1200,8 @@ rta_lookup(rta *o)
 
   r = rta_copy(o);
   r->hash_key = h;
-  r->cached = 1;
+  r->aflags = RTAF_CACHED;
+  rt_lock_source(r->src);
   rt_lock_hostentry(r->hostentry);
   rta_insert(r);
 
@@ -1229,17 +1214,18 @@ rta_lookup(rta *o)
 void
 rta__free(rta *a)
 {
-  ASSERT(rta_cache_count && a->cached);
+  ASSERT(rta_cache_count && (a->aflags & RTAF_CACHED));
   rta_cache_count--;
   *a->pprev = a->next;
   if (a->next)
     a->next->pprev = a->pprev;
   rt_unlock_hostentry(a->hostentry);
+  rt_unlock_source(a->src);
   if (a->nh.next)
     nexthop_free(a->nh.next);
   ea_free(a->eattrs);
-  a->cached = 0;
-  sl_free(a);
+  a->aflags = 0;		/* Poison the entry */
+  sl_free(rta_slab(a), a);
 }
 
 rta *
@@ -1253,7 +1239,7 @@ rta_do_cow(rta *o, linpool *lp)
       memcpy(*nhn, nho, nexthop_size(nho));
       nhn = &((*nhn)->next);
     }
-  r->cached = 0;
+  r->aflags = 0;
   r->uc = 0;
   return r;
 }
@@ -1267,16 +1253,16 @@ rta_do_cow(rta *o, linpool *lp)
 void
 rta_dump(rta *a)
 {
-  static char *rts[] = { "", "RTS_STATIC", "RTS_INHERIT", "RTS_DEVICE",
+  static char *rts[] = { "RTS_DUMMY", "RTS_STATIC", "RTS_INHERIT", "RTS_DEVICE",
 			 "RTS_STAT_DEV", "RTS_REDIR", "RTS_RIP",
 			 "RTS_OSPF", "RTS_OSPF_IA", "RTS_OSPF_EXT1",
 			 "RTS_OSPF_EXT2", "RTS_BGP", "RTS_PIPE", "RTS_BABEL" };
   static char *rtd[] = { "", " DEV", " HOLE", " UNREACH", " PROHIBIT" };
 
-  debug("pref=%d uc=%d %s %s%s h=%04x",
-	a->pref, a->uc, rts[a->source], ip_scope_text(a->scope),
+  debug("p=%s uc=%d %s %s%s h=%04x",
+	a->src->proto->name, a->uc, rts[a->source], ip_scope_text(a->scope),
 	rtd[a->dest], a->hash_key);
-  if (!a->cached)
+  if (!(a->aflags & RTAF_CACHED))
     debug(" !CACHED");
   debug(" <-%I", a->from);
   if (a->dest == RTD_UNICAST)
