@@ -15,8 +15,6 @@
 #include "nest/cli.h"
 #include "nest/iface.h"
 #include "filter/filter.h"
-#include "filter/data.h"
-#include "sysdep/unix/krt.h"
 
 static void
 rt_show_table(struct cli *c, struct rt_show_data *d)
@@ -30,20 +28,14 @@ rt_show_table(struct cli *c, struct rt_show_data *d)
   d->last_table = d->tab;
 }
 
-static inline struct krt_proto *
-rt_show_get_kernel(struct rt_show_data *d)
-{
-  struct proto_config *krt = d->tab->table->config->krt_attached;
-  return krt ? (struct krt_proto *) krt->proto : NULL;
-}
-
 static void
-rt_show_rte(struct cli *c, byte *ia, rte *e, struct rt_show_data *d, int primary)
+rt_show_rte(struct cli *c, byte *ia, rte *e, struct rt_show_data *d)
 {
   byte from[IPA_MAX_TEXT_LENGTH+8];
   byte tm[TM_DATETIME_BUFFER_SIZE], info[256];
   rta *a = e->attrs;
-  int sync_error = d->kernel ? krt_get_sync_error(d->kernel, e) : 0;
+  int primary = (e->net->routes == e);
+  int sync_error = (e->net->n.flags & KRF_SYNC_ERROR);
   void (*get_route_info)(struct rte *, byte *buf);
   struct nexthop *nh;
 
@@ -105,15 +97,11 @@ rt_show_net(struct cli *c, net *n, struct rt_show_data *d)
   rte *e, *ee;
   byte ia[NET_MAX_TEXT_LENGTH+1];
   struct channel *ec = d->tab->export_channel;
-
-  /* The Clang static analyzer complains that ec may be NULL.
-   * It should be ensured to be not NULL by rt_show_prepare_tables() */
-  ASSUME(!d->export_mode || ec);
-
   int first = 1;
-  int first_show = 1;
   int pass = 0;
 
+  bsnprintf(ia, sizeof(ia), "%N", n->n.addr);
+
   for (e = n->routes; e; e = e->next)
     {
       if (rte_is_filtered(e) != d->filtered)
@@ -133,17 +121,9 @@ rt_show_net(struct cli *c, net *n, struct rt_show_data *d)
       if (ec && (ec->export_state == ES_DOWN))
 	goto skip;
 
-      if (d->export_mode == RSEM_EXPORTED)
-        {
-	  if (!bmap_test(&ec->export_map, ee->id))
-	    goto skip;
-
-	  // if (ec->ra_mode != RA_ANY)
-	  //   pass = 1;
-        }
-      else if ((d->export_mode == RSEM_EXPORT) && (ec->ra_mode == RA_MERGED))
+      /* Special case for merged export */
+      if ((d->export_mode == RSEM_EXPORT) && (ec->ra_mode == RA_MERGED))
 	{
-	  /* Special case for merged export */
 	  rte *rt_free;
 	  e = rt_export_merged(ec, n, &rt_free, c->show_pool, 1);
 	  pass = 1;
@@ -187,17 +167,10 @@ rt_show_net(struct cli *c, net *n, struct rt_show_data *d)
 	goto skip;
 
       if (d->stats < 2)
-      {
-	if (first_show)
-	  net_format(n->n.addr, ia, sizeof(ia));
-	else
-	  ia[0] = 0;
-
-	rt_show_rte(c, ia, e, d, (e->net->routes == ee));
-	first_show = 0;
-      }
+	rt_show_rte(c, ia, e, d);
 
       d->show_counter++;
+      ia[0] = 0;
 
     skip:
       if (e != ee)
@@ -219,12 +192,9 @@ rt_show_cleanup(struct cli *c)
   struct rt_show_data_rtable *tab;
 
   /* Unlink the iterator */
-  if (d->table_open && !d->trie_walk)
+  if (d->table_open)
     fit_get(&d->tab->table->fib, &d->fit);
 
-  if (d->walk_lock)
-    rt_unlock_trie(d->tab->table, d->walk_lock);
-
   /* Unlock referenced tables */
   WALK_LIST(tab, d->tables)
     rt_unlock_table(tab->table);
@@ -234,13 +204,12 @@ static void
 rt_show_cont(struct cli *c)
 {
   struct rt_show_data *d = c->rover;
-  struct rtable *tab = d->tab->table;
 #ifdef DEBUGGING
   unsigned max = 4;
 #else
   unsigned max = 64;
 #endif
-  struct fib *fib = &tab->fib;
+  struct fib *fib = &d->tab->table->fib;
   struct fib_iterator *it = &d->fit;
 
   if (d->running_on_config && (d->running_on_config != config))
@@ -251,25 +220,9 @@ rt_show_cont(struct cli *c)
 
   if (!d->table_open)
   {
-    /* We use either trie-based walk or fib-based walk */
-    d->trie_walk = tab->trie &&
-      (d->addr_mode == RSD_ADDR_IN) &&
-      net_val_match(tab->addr_type, NB_IP);
-
-    if (d->trie_walk && !d->walk_state)
-      d->walk_state = lp_allocz(c->parser_pool, sizeof (struct f_trie_walk_state));
-
-    if (d->trie_walk)
-    {
-      d->walk_lock = rt_lock_trie(tab);
-      trie_walk_init(d->walk_state, tab->trie, d->addr);
-    }
-    else
-      FIB_ITERATE_INIT(&d->fit, &tab->fib);
-
+    FIB_ITERATE_INIT(&d->fit, &d->tab->table->fib);
     d->table_open = 1;
     d->table_counter++;
-    d->kernel = rt_show_get_kernel(d);
 
     d->show_counter_last = d->show_counter;
     d->rt_counter_last   = d->rt_counter;
@@ -279,44 +232,16 @@ rt_show_cont(struct cli *c)
       rt_show_table(c, d);
   }
 
-  if (d->trie_walk)
+  FIB_ITERATE_START(fib, it, net, n)
   {
-    /* Trie-based walk */
-    net_addr addr;
-    while (trie_walk_next(d->walk_state, &addr))
+    if (!max--)
     {
-      net *n = net_find(tab, &addr);
-      if (!n)
-	continue;
-
-      rt_show_net(c, n, d);
-
-      if (!--max)
-	return;
+      FIB_ITERATE_PUT(it);
+      return;
     }
-
-    rt_unlock_trie(tab, d->walk_lock);
-    d->walk_lock = NULL;
-  }
-  else
-  {
-    /* fib-based walk */
-    FIB_ITERATE_START(fib, it, net, n)
-    {
-      if ((d->addr_mode == RSD_ADDR_IN) && (!net_in_netX(n->n.addr, d->addr)))
-	goto next;
-
-      if (!max--)
-      {
-	FIB_ITERATE_PUT(it);
-	return;
-      }
-      rt_show_net(c, n, d);
-
-    next:;
-    }
-    FIB_ITERATE_END;
+    rt_show_net(c, n, d);
   }
+  FIB_ITERATE_END;
 
   if (d->stats)
   {
@@ -325,10 +250,9 @@ rt_show_cont(struct cli *c)
 
     cli_printf(c, -1007, "%d of %d routes for %d networks in table %s",
 	       d->show_counter - d->show_counter_last, d->rt_counter - d->rt_counter_last,
-	       d->net_counter - d->net_counter_last, tab->name);
+	       d->net_counter - d->net_counter_last, d->tab->table->name);
   }
 
-  d->kernel = NULL;
   d->table_open = 0;
   d->tab = NODE_NEXT(d->tab);
 
@@ -456,7 +380,7 @@ rt_show(struct rt_show_data *d)
 
   rt_show_prepare_tables(d);
 
-  if (!d->addr || (d->addr_mode == RSD_ADDR_IN))
+  if (!d->addr)
   {
     WALK_LIST(tab, d->tables)
       rt_lock_table(tab->table);
@@ -472,9 +396,8 @@ rt_show(struct rt_show_data *d)
     WALK_LIST(tab, d->tables)
     {
       d->tab = tab;
-      d->kernel = rt_show_get_kernel(d);
 
-      if (d->addr_mode == RSD_ADDR_FOR)
+      if (d->show_for)
 	n = net_route(tab->table, d->addr);
       else
 	n = net_find(tab->table, d->addr);
