@@ -224,15 +224,12 @@ mrt_next_table_(rtable *tab, rtable *tab_ptr, const char *pattern)
     return !tab ? tab_ptr : NULL;
 
   /* Walk routing_tables list, starting after tab (if non-NULL) */
-  for (node *tn = tab ? tab->n.next : HEAD(routing_tables);
-       NODE_VALID(tn);
-       tn = tn->next)
-  {
-    tab = SKIP_BACK(struct rtable, n, tn);
+  for (tab = !tab ? HEAD(routing_tables) : NODE_NEXT(tab);
+       NODE_VALID(tab);
+       tab = NODE_NEXT(tab))
     if (patmatch(pattern, tab->name) &&
 	((tab->addr_type == NET_IP4) || (tab->addr_type == NET_IP6)))
       return tab;
-  }
 
   return NULL;
 }
@@ -247,6 +244,7 @@ mrt_next_table(struct mrt_table_dump_state *s)
 
   s->table = tab;
   s->ipv4 = tab ? (tab->addr_type == NET_IP4) : 0;
+  s->bws->mp_reach = !s->ipv4;
 
   if (s->table)
     rt_lock_table(s->table);
@@ -419,51 +417,6 @@ mrt_rib_table_header(struct mrt_table_dump_state *s, net_addr *n)
   mrt_put_u16(b, 0);
 }
 
-#ifdef CONFIG_BGP
-static void
-mrt_rib_table_entry_bgp_attrs(struct mrt_table_dump_state *s, rte *r)
-{
-  struct ea_list *eattrs = r->attrs->eattrs;
-  buffer *b = &s->buf;
-
-  if (!eattrs)
-    return;
-
-  /* Attribute list must be normalized for bgp_encode_attrs() */
-  if (!rta_is_cached(r->attrs))
-    ea_normalize(eattrs);
-
-  mrt_buffer_need(b, MRT_ATTR_BUFFER_SIZE);
-  byte *pos = b->pos;
-
-  s->bws->mp_reach = !s->ipv4;
-  s->bws->mp_next_hop = NULL;
-
-  /* Encode BGP attributes */
-  int len = bgp_encode_attrs(s->bws, eattrs, pos, b->end);
-  if (len < 0)
-    goto fail;
-  pos += len;
-
-  /* Encode IPv6 next hop separately as fake MP_REACH_NLRI attribute */
-  if (s->bws->mp_next_hop)
-  {
-    len = bgp_encode_mp_reach_mrt(s->bws, s->bws->mp_next_hop, pos, b->end - pos);
-    if (len < 0)
-      goto fail;
-    pos += len;
-  }
-
-  /* Update attribute length and advance buffer pos */
-  put_u16(b->pos - 2, pos - b->pos);
-  b->pos = pos;
-  return;
-
-fail:
-  mrt_log(s, "Attribute list too long for %N", r->net->n.addr);
-}
-#endif
-
 static void
 mrt_rib_table_entry(struct mrt_table_dump_state *s, rte *r)
 {
@@ -472,9 +425,9 @@ mrt_rib_table_entry(struct mrt_table_dump_state *s, rte *r)
 
 #ifdef CONFIG_BGP
   /* Find peer index */
-  if (r->src->proto->proto == &proto_bgp)
+  if (r->attrs->src->proto->proto == &proto_bgp)
   {
-    struct bgp_proto *p = (void *) r->src->proto;
+    struct bgp_proto *p = (void *) r->attrs->src->proto;
     struct mrt_peer_entry *n =
       HASH_FIND(s->peer_hash, PEER, p->remote_id, p->remote_as, p->remote_ip);
 
@@ -488,13 +441,31 @@ mrt_rib_table_entry(struct mrt_table_dump_state *s, rte *r)
 
   /* Path Identifier */
   if (s->add_path)
-    mrt_put_u32(b, r->src->private_id);
+    mrt_put_u32(b, r->attrs->src->private_id);
 
   /* Route Attributes */
   mrt_put_u16(b, 0);
 
 #ifdef CONFIG_BGP
-  mrt_rib_table_entry_bgp_attrs(s, r);
+  if (r->attrs->eattrs)
+  {
+    struct ea_list *eattrs = r->attrs->eattrs;
+
+    if (!rta_is_cached(r->attrs))
+      ea_normalize(eattrs);
+
+    mrt_buffer_need(b, MRT_ATTR_BUFFER_SIZE);
+    int alen = bgp_encode_attrs(s->bws, eattrs, b->pos, b->end);
+
+    if (alen < 0)
+    {
+      mrt_log(s, "Attribute list too long for %N", r->net->n.addr);
+      alen = 0;
+    }
+
+    put_u16(b->pos - 2, alen);
+    b->pos += alen;
+  }
 #endif
 
   s->entry_count++;
@@ -519,12 +490,14 @@ mrt_rib_table_dump(struct mrt_table_dump_state *s, net *n, int add_path)
       continue;
 
     /* Skip routes that should be reported in the other phase */
-    if (!s->always_add_path && (!rt->src->private_id != !s->add_path))
+    if (!s->always_add_path && (!rt->attrs->src->private_id != !s->add_path))
     {
       s->want_add_path = 1;
       continue;
     }
 
+    rte_make_tmp_attrs(&rt, s->linpool, NULL);
+
     if (f_run(s->filter, &rt, s->linpool, 0) <= F_ACCEPT)
       mrt_rib_table_entry(s, rt);
 
@@ -560,8 +533,8 @@ mrt_table_dump_init(pool *pp)
   struct mrt_table_dump_state *s = mb_allocz(pool, sizeof(struct mrt_table_dump_state));
 
   s->pool = pool;
-  s->linpool = lp_new(pool);
-  s->peer_lp = lp_new(pool);
+  s->linpool = lp_new(pool, 4080);
+  s->peer_lp = lp_new(pool, 4080);
   mrt_buffer_init(&s->buf, pool, 2 * MRT_ATTR_BUFFER_SIZE);
 
   /* We lock the current config as we may reference it indirectly by filter */
@@ -916,9 +889,3 @@ struct protocol proto_mrt = {
   .reconfigure =	mrt_reconfigure,
   .copy_config =	mrt_copy_config,
 };
-
-void
-mrt_build(void)
-{
-  proto_build(&proto_mrt);
-}
