@@ -522,24 +522,21 @@ static inline void
 add_nbma_node(struct ospf_iface *ifa, struct nbma_node *src, int found)
 {
   struct nbma_node *n = mb_alloc(ifa->pool, sizeof(struct nbma_node));
-
-  n->n = (node) {};
   add_tail(&ifa->nbma_list, NODE n);
-
   n->ip = src->ip;
   n->eligible = src->eligible;
   n->found = found;
 }
 
 static int
-ospf_iface_stubby(struct ospf_iface_patt *ip, struct ifa *addr, int type)
+ospf_iface_stubby(struct ospf_iface_patt *ip, struct ifa *addr)
 {
   /* vlink cannot be stub */
-  if (type == OSPF_IT_VLINK)
+  if (ip->type == OSPF_IT_VLINK)
     return 0;
 
-  /* Host address on Broadcast/NBMA */
-  if (((type == OSPF_IT_BCAST) || (type == OSPF_IT_NBMA)) && (addr->flags & IA_HOST))
+  /* a host address */
+  if (addr->flags & IA_HOST)
     return 1;
 
   /* a loopback iface */
@@ -587,7 +584,10 @@ ospf_iface_new(struct ospf_area *oa, struct ifa *addr, struct ospf_iface_patt *i
   ifa->strictnbma = ip->strictnbma;
   ifa->waitint = ip->waitint;
   ifa->deadint = ip->deadint;
+  ifa->stub = ospf_iface_stubby(ip, addr);
   ifa->ioprob = OSPF_I_OK;
+  ifa->tx_length = ifa_tx_length(ifa);
+  ifa->tx_hdrlen = ifa_tx_hdrlen(ifa);
   ifa->check_link = ip->check_link;
   ifa->ecmp_weight = ip->ecmp_weight;
   ifa->check_ttl = (ip->ttl_security == 1);
@@ -596,23 +596,13 @@ ospf_iface_new(struct ospf_area *oa, struct ifa *addr, struct ospf_iface_patt *i
   ifa->passwords = ip->passwords;
   ifa->instance_id = ip->instance_id;
 
-  /* This must be done after relevant fields are set */
-  ifa->tx_length = ifa_tx_length(ifa);
-  ifa->tx_hdrlen = ifa_tx_hdrlen(ifa);
-
-  ifa->ptp_netmask = !(addr->flags & (IA_HOST | IA_PEER));
+  ifa->ptp_netmask = !(addr->flags & IA_PEER);
   if (ip->ptp_netmask < 2)
     ifa->ptp_netmask = ip->ptp_netmask;
 
-  /* For compatibility, we may use ptp_address even for unnumbered links */
-  ifa->ptp_address = !(addr->flags & (IA_HOST | IA_PEER)) || (p->gr_mode != OSPF_GR_ABLE);
-  if (ip->ptp_address < 2)
-    ifa->ptp_address = ip->ptp_address;
-
   ifa->drip = ifa->bdrip = ospf_is_v2(p) ? IPA_NONE4 : IPA_NONE6;
 
   ifa->type = ospf_iface_classify(ip->type, addr);
-  ifa->stub = ospf_iface_stubby(ip, addr, ifa->type);
 
   /* Check validity of interface type */
   int old_type = ifa->type;
@@ -650,7 +640,7 @@ ospf_iface_new(struct ospf_area *oa, struct ifa *addr, struct ospf_iface_patt *i
        should be used). Because OSPFv3 iface is not subnet-specific,
        there is no need for ipa_in_net() check */
 
-    if (ospf_is_v2(p) && !ospf_ipa_local(nb->ip, addr))
+    if (ospf_is_v2(p) && !ipa_in_netX(nb->ip, &addr->prefix))
       continue;
 
     if (ospf_is_v3(p) && !ipa_is_link_local(nb->ip))
@@ -708,14 +698,12 @@ ospf_iface_new_vlink(struct ospf_proto *p, struct ospf_iface_patt *ip)
   ifa->waitint = ip->waitint;
   ifa->deadint = ip->deadint;
   ifa->inftransdelay = ip->inftransdelay;
+  ifa->tx_length = ospf_is_v2(p) ? IP4_MIN_MTU : IP6_MIN_MTU;
+  ifa->tx_hdrlen = ifa_tx_hdrlen(ifa);
   ifa->autype = ip->autype;
   ifa->passwords = ip->passwords;
   ifa->instance_id = ip->instance_id;
 
-  /* This must be done after relevant fields are set */
-  ifa->tx_length = ospf_is_v2(p) ? IP4_MIN_MTU : IP6_MIN_MTU;
-  ifa->tx_hdrlen = ifa_tx_hdrlen(ifa);
-
   ifa->type = OSPF_IT_VLINK;
 
   ifa->state = OSPF_IS_DOWN;
@@ -770,7 +758,7 @@ ospf_iface_reconfigure(struct ospf_iface *ifa, struct ospf_iface_patt *new)
   if (old_type != new_type)
     return 0;
 
-  int new_stub = ospf_iface_stubby(new, ifa->addr, new_type);
+  int new_stub = ospf_iface_stubby(new, ifa->addr);
   if (ifa->stub != new_stub)
     return 0;
 
@@ -932,7 +920,7 @@ ospf_iface_reconfigure(struct ospf_iface *ifa, struct ospf_iface_patt *new)
   WALK_LIST(nb, new->nbma_list)
   {
     /* See related note in ospf_iface_new() */
-    if (ospf_is_v2(p) && !ospf_ipa_local(nb->ip, ifa->addr))
+    if (ospf_is_v2(p) && !ipa_in_netX(nb->ip, &ifa->addr->prefix))
       continue;
 
     if (ospf_is_v3(p) && !ipa_is_link_local(nb->ip))
@@ -1012,29 +1000,6 @@ ospf_iface_reconfigure(struct ospf_iface *ifa, struct ospf_iface_patt *new)
     ospf_notify_link_lsa(ifa);
   }
 
-  /* PtP netmask */
-  int new_ptp_netmask = (new->ptp_netmask < 2) ? new->ptp_netmask :
-    !(ifa->addr->flags & (IA_HOST | IA_PEER));
-  if (ifa->ptp_netmask != new_ptp_netmask)
-  {
-    OSPF_TRACE(D_EVENTS, "Changing PtP netmask option of %s from %d to %d",
-	       ifname, ifa->ptp_netmask, new_ptp_netmask);
-    ifa->ptp_netmask = new_ptp_netmask;
-  }
-
-  /* PtP address */
-  int new_ptp_address = (new->ptp_address < 2) ? new->ptp_address :
-    (!(ifa->addr->flags & (IA_HOST | IA_PEER)) || (p->gr_mode != OSPF_GR_ABLE));
-  if (ifa->ptp_address != new_ptp_address)
-  {
-    /* Keep it silent for implicit changes */
-    if (new->ptp_address < 2)
-      OSPF_TRACE(D_EVENTS, "Changing PtP address option of %s from %d to %d",
-		 ifname, ifa->ptp_address, new_ptp_address);
-
-    ifa->ptp_address = new_ptp_address;
-  }
-
   /* BFD */
   if (ifa->bfd != new->bfd)
   {
