@@ -71,6 +71,7 @@ ospf_install_lsa(struct ospf_proto *p, struct ospf_lsa_header *lsa, u32 type, u3
   en->lsa = *lsa;
   en->init_age = en->lsa.age;
   en->inst_time = current_time();
+  en->gr_dirty = p->gr_recovery && (lsa->rt == p->router_id);
 
   /*
    * We do not set en->mode. It is either default LSA_M_BASIC, or in a special
@@ -83,7 +84,10 @@ ospf_install_lsa(struct ospf_proto *p, struct ospf_lsa_header *lsa, u32 type, u3
 	     en->lsa_type, en->lsa.id, en->lsa.rt, en->lsa.sn, en->lsa.age);
 
   if (change)
+  {
+    ospf_neigh_lsadb_changed(p, en);
     ospf_schedule_rtcalc(p);
+  }
 
   return en;
 }
@@ -243,6 +247,7 @@ ospf_do_originate_lsa(struct ospf_proto *p, struct top_hash_entry *en, void *lsa
   en->lsa.age = 0;
   en->init_age = 0;
   en->inst_time = current_time();
+  en->gr_dirty = 0;
   lsa_generate_checksum(&en->lsa, en->lsa_body);
 
   OSPF_TRACE(D_EVENTS, "Originating LSA: Type: %04x, Id: %R, Rt: %R, Seq: %08x",
@@ -251,7 +256,10 @@ ospf_do_originate_lsa(struct ospf_proto *p, struct top_hash_entry *en, void *lsa
   ospf_flood_lsa(p, en, NULL);
 
   if (en->mode == LSA_M_BASIC)
+  {
+    ospf_neigh_lsadb_changed(p, en);
     ospf_schedule_rtcalc(p);
+  }
 
   return 1;
 }
@@ -273,11 +281,15 @@ ospf_do_originate_lsa(struct ospf_proto *p, struct top_hash_entry *en, void *lsa
 struct top_hash_entry *
 ospf_originate_lsa(struct ospf_proto *p, struct ospf_new_lsa *lsa)
 {
-  struct top_hash_entry *en;
+  struct top_hash_entry *en = NULL;
   void *lsa_body = p->lsab;
   u16 lsa_blen = p->lsab_used;
   u16 lsa_length = sizeof(struct ospf_lsa_header) + lsa_blen;
 
+  /* RFC 3623 2 (1) - do not originate topology LSAs during graceful restart */
+  if (p->gr_recovery && (LSA_FUNCTION(lsa->type) <= LSA_FUNCTION(LSA_T_NSSA)))
+    goto drop;
+
   /* For OSPFv2 Opaque LSAs, LS ID consists of Opaque Type and Opaque ID */
   if (ospf_is_v2(p) && lsa_is_opaque(lsa->type))
     lsa->id |= (u32) lsa_get_opaque_type(lsa->type) << 24;
@@ -321,7 +333,8 @@ ospf_originate_lsa(struct ospf_proto *p, struct ospf_new_lsa *lsa)
   if ((en->lsa.age < LSA_MAXAGE) &&
       (lsa_length == en->lsa.length) &&
       !memcmp(lsa_body, en->lsa_body, lsa_blen) &&
-      (!ospf_is_v2(p) || (lsa->opts == lsa_get_options(&en->lsa))))
+      (!ospf_is_v2(p) || (lsa->opts == lsa_get_options(&en->lsa))) &&
+      !en->gr_dirty)
     goto drop;
 
   lsa_body = lsab_flush(p);
@@ -414,6 +427,7 @@ void
 ospf_flush_lsa(struct ospf_proto *p, struct top_hash_entry *en)
 {
   en->nf = NULL;
+  en->gr_dirty = 0;
 
   if (en->next_lsa_body)
   {
@@ -433,7 +447,10 @@ ospf_flush_lsa(struct ospf_proto *p, struct top_hash_entry *en)
   ospf_flood_lsa(p, en, NULL);
 
   if (en->mode == LSA_M_BASIC)
+  {
+    ospf_neigh_lsadb_changed(p, en);
     ospf_schedule_rtcalc(p);
+  }
 
   en->mode = LSA_M_BASIC;
 }
@@ -525,6 +542,29 @@ ospf_update_lsadb(struct ospf_proto *p)
   }
 }
 
+void
+ospf_feed_begin(struct channel *C, int initial UNUSED)
+{
+  struct ospf_proto *p = (struct ospf_proto *) C->proto;
+  struct top_hash_entry *en;
+
+  /* Mark all external LSAs as stale */
+  WALK_SLIST(en, p->lsal)
+    if (en->mode == LSA_M_EXPORT)
+      en->mode = LSA_M_EXPORT_STALE;
+}
+
+void
+ospf_feed_end(struct channel *C)
+{
+  struct ospf_proto *p = (struct ospf_proto *) C->proto;
+  struct top_hash_entry *en;
+
+  /* Flush stale LSAs */
+  WALK_SLIST(en, p->lsal)
+    if (en->mode == LSA_M_EXPORT_STALE)
+      ospf_flush_lsa(p, en);
+}
 
 static u32
 ort_to_lsaid(struct ospf_proto *p, ort *nf)
@@ -1424,6 +1464,7 @@ prepare_prefix_rt_lsa_body(struct ospf_proto *p, struct ospf_area *oa)
   struct ospf_config *cf = (struct ospf_config *) (p->p.cf);
   struct ospf_iface *ifa;
   struct ospf_lsa_prefix *lp;
+  uint max = ospf_is_ip4(p) ? IP4_MAX_PREFIX_LENGTH : IP6_MAX_PREFIX_LENGTH;
   int host_addr = 0;
   int net_lsa;
   int i = 0;
@@ -1457,7 +1498,7 @@ prepare_prefix_rt_lsa_body(struct ospf_proto *p, struct ospf_area *oa)
 	  (a->scope <= SCOPE_LINK))
 	continue;
 
-      if (((a->prefix.pxlen < IP6_MAX_PREFIX_LENGTH) && net_lsa) ||
+      if (((a->prefix.pxlen < max) && net_lsa) ||
 	  configured_stubnet(oa, a))
 	continue;
 
@@ -1465,8 +1506,13 @@ prepare_prefix_rt_lsa_body(struct ospf_proto *p, struct ospf_area *oa)
 	  (ifa->state == OSPF_IS_LOOP) ||
 	  (ifa->type == OSPF_IT_PTMP))
       {
-	net_addr_ip6 net = NET_ADDR_IP6(a->ip, IP6_MAX_PREFIX_LENGTH);
-	lsab_put_prefix(p, (net_addr *) &net, 0);
+	net_addr net;
+	if (a->prefix.type == NET_IP4)
+	  net_fill_ip4(&net, ipa_to_ip4(a->ip), IP4_MAX_PREFIX_LENGTH);
+	else
+	  net_fill_ip6(&net, ipa_to_ip6(a->ip), IP6_MAX_PREFIX_LENGTH);
+
+	lsab_put_prefix(p, &net, 0);
 	host_addr = 1;
       }
       else
@@ -1482,7 +1528,7 @@ prepare_prefix_rt_lsa_body(struct ospf_proto *p, struct ospf_area *oa)
     if (!sn->hidden)
     {
       lsab_put_prefix(p, &sn->prefix, sn->cost);
-      if (sn->prefix.pxlen == IP6_MAX_PREFIX_LENGTH)
+      if (sn->prefix.pxlen == max)
 	host_addr = 1;
       i++;
     }
@@ -1669,6 +1715,59 @@ ospf_originate_prefix_net_lsa(struct ospf_proto *p, struct ospf_iface *ifa)
 }
 
 
+/*
+ *	Grace LSA handling
+ *	Type = LSA_T_GR, opaque type = LSA_OT_GR
+ */
+
+static inline void
+ospf_add_gr_period_tlv(struct ospf_proto *p, uint period)
+{
+  struct ospf_tlv *tlv = lsab_allocz(p, sizeof(struct ospf_tlv) + sizeof(u32));
+  tlv->type = LSA_GR_PERIOD;
+  tlv->length = 4;
+  tlv->data[0] = period;
+}
+
+static inline void
+ospf_add_gr_reason_tlv(struct ospf_proto *p, uint reason)
+{
+  struct ospf_tlv *tlv = lsab_allocz(p, sizeof(struct ospf_tlv) + sizeof(u32));
+  tlv->type = LSA_GR_REASON;
+  tlv->length = 1;
+  tlv->data[0] = reason << 24;
+}
+
+static inline void
+ospf_add_gr_address_tlv(struct ospf_proto *p, ip4_addr addr)
+{
+  struct ospf_tlv *tlv = lsab_allocz(p, sizeof(struct ospf_tlv) + sizeof(u32));
+  tlv->type = LSA_GR_ADDRESS;
+  tlv->length = 4;
+  tlv->data[0] = ip4_to_u32(addr);
+}
+
+void
+ospf_originate_gr_lsa(struct ospf_proto *p, struct ospf_iface *ifa)
+{
+  struct ospf_new_lsa lsa = {
+    .type = LSA_T_GR,
+    .dom  = ifa->iface_id,
+    .id   = ospf_is_v2(p) ? 0 : ifa->iface_id,
+    .ifa  = ifa
+  };
+
+  ospf_add_gr_period_tlv(p, p->gr_time);
+  ospf_add_gr_reason_tlv(p, 0);
+
+  uint t = ifa->type;
+  if (ospf_is_v2(p) && ((t == OSPF_IT_BCAST) || (t == OSPF_IT_NBMA) || (t == OSPF_IT_PTMP)))
+    ospf_add_gr_address_tlv(p, ipa_to_ip4(ifa->addr->ip));
+
+  ospf_originate_lsa(p, &lsa);
+}
+
+
 /*
  *	Router Information LSA handling
  *	Type = LSA_T_RI_AREA, opaque type = LSA_OT_RI
@@ -1712,6 +1811,10 @@ ospf_update_topology(struct ospf_proto *p)
   struct ospf_area *oa;
   struct ospf_iface *ifa;
 
+  /* No LSA reorigination during GR recovery */
+  if (p->gr_recovery)
+    return;
+
   WALK_LIST(oa, p->area_list)
   {
     if (oa->update_rt_lsa)
