@@ -43,10 +43,6 @@
 
 #include "pipe.h"
 
-#ifdef CONFIG_BGP
-#include "proto/bgp/bgp.h"
-#endif
-
 static void
 pipe_rt_notify(struct proto *P, struct channel *src_ch, net *n, rte *new, rte *old)
 {
@@ -69,26 +65,31 @@ pipe_rt_notify(struct proto *P, struct channel *src_ch, net *n, rte *new, rte *o
 
   if (new)
     {
-      src = new->src;
-
       a = alloca(rta_size(new->attrs));
       memcpy(a, new->attrs, rta_size(new->attrs));
 
-      a->cached = 0;
+      a->aflags = 0;
       a->hostentry = NULL;
-      e = rte_get_temp(a, src);
+      e = rte_get_temp(a);
+      e->pflags = 0;
+
+      /* Copy protocol specific embedded attributes. */
+      memcpy(&(e->u), &(new->u), sizeof(e->u));
+      e->pref = new->pref;
       e->pflags = new->pflags;
 
 #ifdef CONFIG_BGP
       /* Hack to cleanup cached value */
-      if (e->src->proto->proto == &proto_bgp)
-	e->pflags &= ~(BGP_REF_STALE | BGP_REF_NOT_STALE);
+      if (e->attrs->src->proto->proto == &proto_bgp)
+	e->u.bgp.stale = -1;
 #endif
+
+      src = a->src;
     }
   else
     {
       e = NULL;
-      src = old->src;
+      src = old->attrs->src;
     }
 
   src_ch->table->pipe_busy = 1;
@@ -97,11 +98,11 @@ pipe_rt_notify(struct proto *P, struct channel *src_ch, net *n, rte *new, rte *o
 }
 
 static int
-pipe_preexport(struct channel *C, rte *e)
+pipe_preexport(struct proto *P, rte **ee, struct linpool *p UNUSED)
 {
-  struct proto *pp = e->sender->proto;
+  struct proto *pp = (*ee)->sender->proto;
 
-  if (pp == C->proto)
+  if (pp == P)
     return -1;	/* Avoid local loops automatically */
 
   return 0;
@@ -140,8 +141,6 @@ pipe_postconfig(struct proto_config *CF)
 
   if (cc->in_keep_filtered)
     cf_error("Pipe protocol prohibits keeping filtered routes");
-
-  cc->debug = cf->c.debug;
 }
 
 static int
@@ -155,9 +154,7 @@ pipe_configure_channels(struct pipe_proto *p, struct pipe_config *cf)
     .table = cc->table,
     .out_filter = cc->out_filter,
     .in_limit = cc->in_limit,
-    .ra_mode = RA_ANY,
-    .debug = cc->debug,
-    .rpki_reload = cc->rpki_reload,
+    .ra_mode = RA_ANY
   };
 
   struct channel_config sec_cf = {
@@ -166,9 +163,7 @@ pipe_configure_channels(struct pipe_proto *p, struct pipe_config *cf)
     .table = cf->peer,
     .out_filter = cc->in_filter,
     .in_limit = cc->out_limit,
-    .ra_mode = RA_ANY,
-    .debug = cc->debug,
-    .rpki_reload = cc->rpki_reload,
+    .ra_mode = RA_ANY
   };
 
   return
@@ -281,14 +276,6 @@ pipe_show_proto_info(struct proto *P)
     pipe_show_stats(p);
 }
 
-void
-pipe_update_debug(struct proto *P)
-{
-  struct pipe_proto *p = (void *) P;
-
-  p->pri->debug = p->sec->debug = p->p.debug;
-}
-
 
 struct protocol proto_pipe = {
   .name =		"Pipe",
@@ -303,9 +290,3 @@ struct protocol proto_pipe = {
   .get_status = 	pipe_get_status,
   .show_proto_info = 	pipe_show_proto_info
 };
-
-void
-pipe_build(void)
-{
-  proto_build(&proto_pipe);
-}
