@@ -9,6 +9,8 @@
  *	Can be freely distributed and used under the terms of the GNU GPL.
  */
 
+#undef LOCAL_DEBUG
+
 #include "rip.h"
 #include "lib/mac.h"
 
@@ -56,13 +58,17 @@ struct rip_block_auth
 {
   u16 must_be_ffff;
   u16 auth_type;
-  char password[0];
-  u16 packet_len;
-  u8 key_id;
-  u8 auth_len;
-  u32 seq_num;
-  u32 unused1;
-  u32 unused2;
+  union {
+    char password[16];
+    struct {
+      u16 packet_len;
+      u8 key_id;
+      u8 auth_len;
+      u32 seq_num;
+      u32 unused1;
+      u32 unused2;
+    };
+  };
 };
 
 /* Authentication tail, RFC 4822 */
@@ -76,8 +82,7 @@ struct rip_auth_tail
 /* Internal representation of RTE block data */
 struct rip_block
 {
-  ip_addr prefix;
-  int pxlen;
+  net_addr net;
   u32 metric;
   u16 tag;
   u16 no_af;
@@ -106,30 +111,30 @@ static inline uint rip_pkt_hdrlen(struct rip_iface *ifa)
 { return sizeof(struct rip_packet) + (ifa->cf->auth_type ? RIP_BLOCK_LENGTH : 0); }
 
 static inline void
-rip_put_block(struct rip_proto *p UNUSED4 UNUSED6, byte *pos, struct rip_block *rte)
+rip_put_block(struct rip_proto *p, byte *pos, struct rip_block *rte)
 {
   if (rip_is_v2(p))
   {
     struct rip_block_v2 *block = (void *) pos;
     block->family = rte->no_af ? 0 : htons(RIP_AF_IPV4);
     block->tag = htons(rte->tag);
-    block->network = ip4_hton(ipa_to_ip4(rte->prefix));
-    block->netmask = ip4_hton(ip4_mkmask(rte->pxlen));
+    block->network = ip4_hton(net4_prefix(&rte->net));
+    block->netmask = ip4_hton(ip4_mkmask(net4_pxlen(&rte->net)));
     block->next_hop = ip4_hton(ipa_to_ip4(rte->next_hop));
     block->metric = htonl(rte->metric);
   }
   else /* RIPng */
   {
     struct rip_block_ng *block = (void *) pos;
-    block->prefix = ip6_hton(ipa_to_ip6(rte->prefix));
+    block->prefix = ip6_hton(net6_prefix(&rte->net));
     block->tag = htons(rte->tag);
-    block->pxlen = rte->pxlen;
+    block->pxlen = net6_pxlen(&rte->net);
     block->metric = rte->metric;
   }
 }
 
 static inline void
-rip_put_next_hop(struct rip_proto *p UNUSED, byte *pos, struct rip_block *rte UNUSED4)
+rip_put_next_hop(struct rip_proto *p UNUSED, byte *pos, struct rip_block *rte)
 {
   struct rip_block_ng *block = (void *) pos;
   block->prefix = ip6_hton(ipa_to_ip6(rte->next_hop));
@@ -139,7 +144,7 @@ rip_put_next_hop(struct rip_proto *p UNUSED, byte *pos, struct rip_block *rte UN
 }
 
 static inline int
-rip_get_block(struct rip_proto *p UNUSED4 UNUSED6, byte *pos, struct rip_block *rte)
+rip_get_block(struct rip_proto *p, byte *pos, struct rip_block *rte)
 {
   if (rip_is_v2(p))
   {
@@ -149,8 +154,8 @@ rip_get_block(struct rip_proto *p UNUSED4 UNUSED6, byte *pos, struct rip_block *
     if (block->family != (rte->no_af ? 0 : htons(RIP_AF_IPV4)))
       return 0;
 
-    rte->prefix = ipa_from_ip4(ip4_ntoh(block->network));
-    rte->pxlen = ip4_masklen(ip4_ntoh(block->netmask));
+    uint pxlen = ip4_masklen(ip4_ntoh(block->netmask));
+    net_fill_ip4(&rte->net, ip4_ntoh(block->network), pxlen);
     rte->metric = ntohl(block->metric);
     rte->tag = ntohs(block->tag);
     rte->next_hop = ipa_from_ip4(ip4_ntoh(block->next_hop));
@@ -169,8 +174,8 @@ rip_get_block(struct rip_proto *p UNUSED4 UNUSED6, byte *pos, struct rip_block *
       return 0;
     }
 
-    rte->prefix = ipa_from_ip6(ip6_ntoh(block->prefix));
-    rte->pxlen = block->pxlen;
+    uint pxlen = (block->pxlen <= IP6_MAX_PREFIX_LENGTH) ? block->pxlen : 255;
+    net_fill_ip6(&rte->net, ip6_ntoh(block->prefix), pxlen);
     rte->metric = block->metric;
     rte->tag = ntohs(block->tag);
     /* rte->next_hop is deliberately kept unmodified */;
@@ -188,7 +193,10 @@ rip_update_csn(struct rip_proto *p UNUSED, struct rip_iface *ifa)
    * have the same CSN. We are using real time, but enforcing monotonicity.
    */
   if (ifa->cf->auth_type == RIP_AUTH_CRYPTO)
-    ifa->csn = (ifa->csn < (u32) now_real) ? (u32) now_real : ifa->csn + 1;
+  {
+    u32 now_real = (u32) (current_real_time() TO_S);
+    ifa->csn = (ifa->csn < now_real) ? now_real : ifa->csn + 1;
+  }
 }
 
 static void
@@ -406,8 +414,9 @@ rip_receive_request(struct rip_proto *p, struct rip_iface *ifa, struct rip_packe
   if (!rip_get_block(p, pos, &b))
     return;
 
-  /* Special case - zero prefix, infinity metric */
-  if (ipa_nonzero(b.prefix) || b.pxlen || (b.metric != p->infinity))
+  /* Special case - infinity metric, for RIPng also zero prefix */
+  if ((b.metric != p->infinity) ||
+      (rip_is_ng(p) && !net_zero_ip6((net_addr_ip6 *) &b.net)))
     return;
 
   /* We do nothing if TX is already active */
@@ -432,6 +441,7 @@ rip_send_response(struct rip_proto *p, struct rip_iface *ifa)
   byte *max = rip_tx_buffer(ifa) + ifa->tx_plen -
     (rip_is_v2(p) ? RIP_BLOCK_LENGTH : 2*RIP_BLOCK_LENGTH);
   ip_addr last_next_hop = IPA_NONE;
+  btime now_ = current_time();
   int send = 0;
 
   struct rip_packet *pkt = (void *) pos;
@@ -440,17 +450,15 @@ rip_send_response(struct rip_proto *p, struct rip_iface *ifa)
   pkt->unused = 0;
   pos += rip_pkt_hdrlen(ifa);
 
-  FIB_ITERATE_START(&p->rtable, &ifa->tx_fit, z)
+  FIB_ITERATE_START(&p->rtable, &ifa->tx_fit, struct rip_entry, en)
   {
-    struct rip_entry *en = (struct rip_entry *) z;
-
     /* Dummy entries */
     if (!en->valid)
       goto next_entry;
 
     /* Stale entries that should be removed */
     if ((en->valid == RIP_ENTRY_STALE) &&
-	((en->changed + ifa->cf->garbage_time) <= now))
+	((en->changed + ifa->cf->garbage_time) <= now_))
       goto next_entry;
 
     /* Triggered updates */
@@ -460,28 +468,28 @@ rip_send_response(struct rip_proto *p, struct rip_iface *ifa)
     /* Not enough space for current entry */
     if (pos > max)
     {
-      FIB_ITERATE_PUT(&ifa->tx_fit, z);
+      FIB_ITERATE_PUT(&ifa->tx_fit);
       goto break_loop;
     }
 
     struct rip_block rte = {
-      .prefix = en->n.prefix,
-      .pxlen = en->n.pxlen,
       .metric = en->metric,
       .tag = en->tag
     };
 
+    net_copy(&rte.net, en->n.addr);
+
     if (en->iface == ifa->iface)
       rte.next_hop = en->next_hop;
 
     if (rip_is_v2(p) && (ifa->cf->version == RIP_V1))
     {
       /* Skipping subnets (i.e. not hosts, classful networks or default route) */
-      if (ip4_masklen(ip4_class_mask(ipa_to_ip4(en->n.prefix))) != en->n.pxlen)
+      if (ip4_masklen(ip4_class_mask(net4_prefix(&rte.net))) != rte.net.pxlen)
 	goto next_entry;
 
       rte.tag = 0;
-      rte.pxlen = 0;
+      rte.net.pxlen = 0;
       rte.next_hop = IPA_NONE;
     }
 
@@ -497,7 +505,7 @@ rip_send_response(struct rip_proto *p, struct rip_iface *ifa)
 	goto next_entry;
     }
 
-    // TRACE(D_PACKETS, "    %I/%d -> %I metric %d", rte.prefix, rte.pxlen, rte.next_hop, rte.metric);
+    // TRACE(D_PACKETS, "    %N -> %I metric %d", &rte.net, rte.next_hop, rte.metric);
 
     /* RIPng next hop entry */
     if (rip_is_ng(p) && !ipa_equal(rte.next_hop, last_next_hop))
@@ -513,7 +521,7 @@ rip_send_response(struct rip_proto *p, struct rip_iface *ifa)
 
   next_entry: ;
   }
-  FIB_ITERATE_END(z);
+  FIB_ITERATE_END;
   ifa->tx_active = 0;
 
   /* Do not send empty packet */
@@ -540,9 +548,9 @@ break_loop:
  * activating the new one.
  */
 void
-rip_send_table(struct rip_proto *p, struct rip_iface *ifa, ip_addr addr, bird_clock_t changed)
+rip_send_table(struct rip_proto *p, struct rip_iface *ifa, ip_addr addr, btime changed)
 {
-  DBG("RIP: Opening TX session to %I on %s\n", dst, ifa->iface->name);
+  DBG("RIP: Opening TX session to %I on %s\n", addr, ifa->iface->name);
 
   rip_reset_tx_session(p, ifa);
 
@@ -591,6 +599,7 @@ rip_receive_response(struct rip_proto *p, struct rip_iface *ifa, struct rip_pack
 
   byte *pos = (byte *) pkt + sizeof(struct rip_packet);
   byte *end = (byte *) pkt + plen;
+  btime now_ = current_time();
 
   for (; pos < end; pos += RIP_BLOCK_LENGTH)
   {
@@ -598,34 +607,36 @@ rip_receive_response(struct rip_proto *p, struct rip_iface *ifa, struct rip_pack
     if (!rip_get_block(p, pos, &rte))
       continue;
 
-    int c = ipa_classify_net(rte.prefix);
-    if ((c < 0) || !(c & IADDR_HOST) || ((c & IADDR_SCOPE_MASK) <= SCOPE_LINK))
-      SKIP("invalid prefix");
-
     if (rip_is_v2(p) && (pkt->version == RIP_V1))
     {
-      if (ifa->cf->check_zero && (rte.tag || rte.pxlen || ipa_nonzero(rte.next_hop)))
+      if (ifa->cf->check_zero && (rte.tag || rte.net.pxlen || ipa_nonzero(rte.next_hop)))
 	SKIP("RIPv1 reserved field is nonzero");
 
       rte.tag = 0;
-      rte.pxlen = ip4_masklen(ip4_class_mask(ipa_to_ip4(rte.prefix)));
+      rte.net.pxlen = ip4_masklen(ip4_class_mask(net4_prefix(&rte.net)));
       rte.next_hop = IPA_NONE;
     }
 
-    if ((rte.pxlen < 0) || (rte.pxlen > MAX_PREFIX_LENGTH))
+    if (rte.net.pxlen == 255)
       SKIP("invalid prefix length");
 
+    net_normalize(&rte.net);
+
+    int c = net_classify(&rte.net);
+    if ((c < 0) || !(c & IADDR_HOST) || ((c & IADDR_SCOPE_MASK) <= SCOPE_LINK))
+      SKIP("invalid prefix");
+
     if (rte.metric > p->infinity)
       SKIP("invalid metric");
 
     if (ipa_nonzero(rte.next_hop))
     {
-      neighbor *nbr = neigh_find2(&p->p, &rte.next_hop, ifa->iface, 0);
+      neighbor *nbr = neigh_find(&p->p, rte.next_hop, ifa->iface, 0);
       if (!nbr || (nbr->scope <= 0))
 	rte.next_hop = IPA_NONE;
     }
 
-    // TRACE(D_PACKETS, "    %I/%d -> %I metric %d", rte.prefix, rte.pxlen, rte.next_hop, rte.metric);
+    // TRACE(D_PACKETS, "    %N -> %I metric %d", &rte.net.n, rte.next_hop, rte.metric);
 
     rte.metric += ifa->cf->metric;
 
@@ -636,19 +647,19 @@ rip_receive_response(struct rip_proto *p, struct rip_iface *ifa, struct rip_pack
 	.next_hop = ipa_nonzero(rte.next_hop) ? rte.next_hop : from->nbr->addr,
 	.metric = rte.metric,
 	.tag = rte.tag,
-	.expires = now + ifa->cf->timeout_time
+	.expires = now_ + ifa->cf->timeout_time
       };
 
-      rip_update_rte(p, &rte.prefix, rte.pxlen, &new);
+      rip_update_rte(p, &rte.net, &new);
     }
     else
-      rip_withdraw_rte(p, &rte.prefix, rte.pxlen, from);
+      rip_withdraw_rte(p, &rte.net, from);
 
     continue;
 
   skip:
-    LOG_RTE("Ignoring route %I/%d received from %I - %s",
-	    rte.prefix, rte.pxlen, from->nbr->addr, err_dsc);
+    LOG_RTE("Ignoring route %N received from %I - %s",
+	    &rte.net, from->nbr->addr, err_dsc);
   }
 }
 
@@ -667,8 +678,7 @@ rip_rx_hook(sock *sk, uint len)
       sk->iface->name, sk->faddr, sk->laddr);
 
   /* Silently ignore my own packets */
-  /* FIXME: Better local address check */
-  if (ipa_equal(ifa->iface->addr->ip, sk->faddr))
+  if (ipa_equal(sk->faddr, sk->saddr))
     return 1;
 
   if (rip_is_ng(p) && !ipa_is_link_local(sk->faddr))
@@ -704,7 +714,7 @@ rip_rx_hook(sock *sk, uint len)
   if ((plen - sizeof(struct rip_packet)) % RIP_BLOCK_LENGTH)
     DROP("invalid length", plen);
 
-  n->last_seen = now;
+  n->last_seen = current_time();
   rip_update_bfd(p, n);
 
   switch (pkt->command)
@@ -736,10 +746,11 @@ rip_open_socket(struct rip_iface *ifa)
 
   sock *sk = sk_new(p->p.pool);
   sk->type = SK_UDP;
+  sk->subtype = rip_is_v2(p) ? SK_IPV4 : SK_IPV6;
   sk->sport = ifa->cf->port;
   sk->dport = ifa->cf->port;
   sk->iface = ifa->iface;
-  sk->saddr = rip_is_v2(p) ? ifa->iface->addr->ip : ifa_llv6(ifa->iface)->ip;
+  sk->saddr = rip_is_v2(p) ? ifa->iface->addr4->ip : ifa->iface->llv6->ip;
   sk->vrf = p->p.vrf;
 
   sk->rx_hook = rip_rx_hook;
