@@ -190,10 +190,6 @@ static inline void
 sockaddr_fill_dl(struct sockaddr_dl *sa, struct iface *ifa)
 {
   uint len = OFFSETOF(struct sockaddr_dl, sdl_data);
-
-  /* Workaround for FreeBSD 13.0 */
-  len = MAX(len, sizeof(struct sockaddr));
-
   memset(sa, 0, len);
   sa->sdl_len = len;
   sa->sdl_family = AF_LINK;
@@ -351,7 +347,7 @@ krt_send_route(struct krt_proto *p, int cmd, rte *e)
 }
 
 void
-krt_replace_rte(struct krt_proto *p, net *n UNUSED, rte *new, rte *old)
+krt_replace_rte(struct krt_proto *p, net *n, rte *new, rte *old)
 {
   int err = 0;
 
@@ -361,37 +357,10 @@ krt_replace_rte(struct krt_proto *p, net *n UNUSED, rte *new, rte *old)
   if (new)
     err = krt_send_route(p, RTM_ADD, new);
 
-  if (new)
-  {
-    if (err < 0)
-      bmap_clear(&p->sync_map, new->id);
-    else
-      bmap_set(&p->sync_map, new->id);
-  }
-}
-
-/**
- * krt_assume_onlink - check if routes on interface are considered onlink
- * @iface: The interface of the next hop
- * @ipv6: Switch to only consider IPv6 or IPv4 addresses.
- *
- * The BSD kernel does not support an onlink flag. If the interface has only
- * host addresses configured, all routes should be considered as onlink and
- * the function returns 1.
- */
-static int
-krt_assume_onlink(struct iface *iface, int ipv6)
-{
-  const u8 type = ipv6 ? NET_IP6 : NET_IP4;
-
-  struct ifa *ifa;
-  WALK_LIST(ifa, iface->addrs)
-  {
-    if ((ifa->prefix.type == type) && !(ifa->flags & IA_HOST))
-      return 0;
-  }
-
-  return 1;
+  if (err < 0)
+    n->n.flags |= KRF_SYNC_ERROR;
+  else
+    n->n.flags &= ~KRF_SYNC_ERROR;
 }
 
 #define SKIP(ARG...) do { DBG("KRT: Ignoring route - " ARG); return; } while(0)
@@ -481,7 +450,7 @@ krt_read_route(struct ks_msg *msg, struct krt_proto *p, int scan)
     net_fill_ip4(&ndst, ipa_to_ip4(idst), pxlen);
 
   if ((flags & RTF_GATEWAY) && ipa_zero(igate))
-    { log(L_ERR "%s (%N) - missing gateway", errmsg, &ndst); return; }
+    { log(L_ERR "%s (%N) - missing gateway", errmsg, ndst); return; }
 
   u32 self_mask = RTF_PROTO1;
   u32 alien_mask = RTF_STATIC | RTF_PROTO1 | RTF_GATEWAY;
@@ -523,6 +492,7 @@ krt_read_route(struct ks_msg *msg, struct krt_proto *p, int scan)
   net = net_get(p->p.main_channel->table, &ndst);
 
   rta a = {
+    .src = p->p.main_source,
     .source = RTS_INHERIT,
     .scope = SCOPE_UNIVERSE,
   };
@@ -555,21 +525,15 @@ krt_read_route(struct ks_msg *msg, struct krt_proto *p, int scan)
   a.dest = RTD_UNICAST;
   if (flags & RTF_GATEWAY)
   {
+    neighbor *ng;
     a.nh.gw = igate;
 
     /* Clean up embedded interface ID returned in link-local address */
     if (ipa_is_link_local(a.nh.gw))
       _I0(a.nh.gw) = 0xfe800000;
 
-    /* The BSD kernel does not support an onlink flag. We heuristically
-       set the onlink flag, if the iface has only host addresses. */
-    if (krt_assume_onlink(a.nh.iface, ipv6))
-      a.nh.flags |= RNF_ONLINK;
-
-    neighbor *nbr;
-    nbr = neigh_find(&p->p, a.nh.gw, a.nh.iface,
-		    (a.nh.flags & RNF_ONLINK) ? NEF_ONLINK : 0);
-    if (!nbr || (nbr->scope == SCOPE_HOST))
+    ng = neigh_find(&p->p, a.nh.gw, a.nh.iface, 0);
+    if (!ng || (ng->scope == SCOPE_HOST))
       {
 	/* Ignore routes with next-hop 127.0.0.1, host routes with such
 	   next-hop appear on OpenBSD for address aliases. */
@@ -583,23 +547,18 @@ krt_read_route(struct ks_msg *msg, struct krt_proto *p, int scan)
   }
 
  done:
-  e = rte_get_temp(&a, p->p.main_source);
+  e = rte_get_temp(&a);
   e->net = net;
-
-  ea_list *ea = alloca(sizeof(ea_list) + 1 * sizeof(eattr));
-  *ea = (ea_list) { .count = 1, .next = e->attrs->eattrs };
-  e->attrs->eattrs = ea;
-
-  ea->attrs[0] = (eattr) {
-    .id = EA_KRT_SOURCE,
-    .type = EAF_TYPE_INT,
-    .u.data = src2,
-  };
+  e->u.krt.src = src;
+  e->u.krt.proto = src2;
+  e->u.krt.seen = 0;
+  e->u.krt.best = 0;
+  e->u.krt.metric = 0;
 
   if (scan)
-    krt_got_route(p, e, src);
+    krt_got_route(p, e);
   else
-    krt_got_route_async(p, e, new, src);
+    krt_got_route_async(p, e, new);
 }
 
 static void
@@ -703,9 +662,6 @@ krt_read_ifinfo(struct ks_msg *msg, int scan)
   else
     f.flags |= IF_MULTIACCESS;      /* NBMA */
 
-  if (fl & IFF_MULTICAST)
-    f.flags |= IF_MULTICAST;
-
   iface = if_update(&f);
 
   if (!scan)
@@ -1206,7 +1162,7 @@ kif_update_sysdep_addr(struct iface *i)
     return 0;
 
   ip4_addr old = i->sysdep;
-  i->sysdep = ipa_to_ip4(ipa_from_sa4((sockaddr *) &ifr.ifr_addr));
+  i->sysdep = ipa_to_ip4(ipa_from_sa4(&ifr.ifr_addr));
 
   return !ip4_equal(i->sysdep, old);
 }
