@@ -43,12 +43,12 @@
  */
 
 #define INIT_MREQ4(maddr,ifa) \
-  { .imr_multiaddr = ipa_to_in4(maddr), .imr_interface = ipa_to_in4(ifa->addr->ip) }
+  { .imr_multiaddr = ipa_to_in4(maddr), .imr_interface = ip4_to_in4(ifa->sysdep) }
 
 static inline int
 sk_setup_multicast4(sock *s)
 {
-  struct in_addr ifa = ipa_to_in4(s->iface->addr->ip);
+  struct in_addr ifa = ip4_to_in4(s->iface->sysdep);
   u8 ttl = s->ttl;
   u8 n = 0;
 
@@ -206,7 +206,7 @@ sk_prepare_ip_header(sock *s, void *hdr, int dlen)
 
 #if defined(__FreeBSD__)
 #define USE_MD5SIG_SETKEY
-#include "lib/setkey.h"
+#include "sysdep/bsd/setkey.h"
 #endif
 
 int
