@@ -19,6 +19,15 @@ struct kif_proto;
 #include "sysdep/config.h"
 #include CONFIG_INCLUDE_KRTSYS_H
 
+/* Flags stored in net->n.flags, rest are in nest/route.h */
+
+#define KRF_VERDICT_MASK 0x0f
+#define KRF_CREATE 0			/* Not seen in kernel table */
+#define KRF_SEEN 1			/* Seen in kernel table during last scan */
+#define KRF_UPDATE 2			/* Need to update this entry */
+#define KRF_DELETE 3			/* Should be deleted */
+#define KRF_IGNORE 4			/* To be ignored */
+
 #define KRT_DEFAULT_ECMP_LIMIT	16
 
 #define EA_KRT_SOURCE	EA_CODE(PROTOCOL_KERNEL, 0)
@@ -49,15 +58,13 @@ struct krt_proto {
   struct krt_state sys;		/* Sysdep state */
 
 #ifdef KRT_ALLOW_LEARN
-  struct rtable *krt_table;	/* Internal table of inherited routes */
+  struct rtable krt_table;	/* Internal table of inherited routes */
 #endif
 
 #ifndef CONFIG_ALL_TABLES_AT_ONCE
   timer *scan_timer;
 #endif
 
-  struct bmap sync_map;		/* Keeps track which exported routes were successfully written to kernel */
-  struct bmap seen_map;		/* Routes seen during last periodic scan */
   node krt_node;		/* Node in krt_proto_list */
   byte af;			/* Kernel address family (AF_*) */
   byte ready;			/* Initial feed has been finished */
@@ -79,14 +86,6 @@ void kif_request_scan(void);
 void krt_got_route(struct krt_proto *p, struct rte *e);
 void krt_got_route_async(struct krt_proto *p, struct rte *e, int new);
 
-static inline int
-krt_get_sync_error(struct krt_proto *p, struct rte *e)
-{
-  return (p->p.proto_state == PS_UP) &&
-    bmap_test(&p->p.main_channel->export_map, e->id) &&
-    !bmap_test(&p->sync_map, e->id);
-}
-
 /* Values for rte->u.krt_sync.src */
 #define KRT_SRC_UNKNOWN	-1	/* Nobody knows */
 #define KRT_SRC_BIRD	 0	/* Our route (not passed in async mode) */
@@ -141,7 +140,7 @@ void krt_sys_copy_config(struct krt_config *, struct krt_config *);
 int  krt_capable(rte *e);
 void krt_do_scan(struct krt_proto *);
 void krt_replace_rte(struct krt_proto *p, net *n, rte *new, rte *old);
-int krt_sys_get_attr(const eattr *a, byte *buf, int buflen);
+int krt_sys_get_attr(eattr *a, byte *buf, int buflen);
 
 
 /* kif sysdep */
