@@ -309,15 +309,22 @@ die(const char *msg, ...)
 void
 debug(const char *msg, ...)
 {
-#define MAX_DEBUG_BUFSIZE 16384
+#define MAX_DEBUG_BUFSIZE       65536
   va_list args;
-  char buf[MAX_DEBUG_BUFSIZE];
+  static uint bufsize = 4096;
+  static char *buf = NULL;
+
+  if (!buf)
+    buf = mb_alloc(&root_pool, bufsize);
 
   va_start(args, msg);
   if (dbgf)
     {
-      if (bvsnprintf(buf, MAX_DEBUG_BUFSIZE, msg, args) < 0)
-	bug("Extremely long debug output, split it.");
+      while (bvsnprintf(buf, bufsize, msg, args) < 0)
+        if (bufsize >= MAX_DEBUG_BUFSIZE)
+          bug("Extremely long debug output, split it.");
+        else
+          buf = mb_realloc(buf, (bufsize *= 2));
 
       fputs(buf, dbgf);
     }
@@ -325,7 +332,7 @@ debug(const char *msg, ...)
 }
 
 static list *
-default_log_list(int initial, const char **syslog_name)
+default_log_list(int initial, char **syslog_name)
 {
   static list log_list;
   init_list(&log_list);
@@ -334,11 +341,7 @@ default_log_list(int initial, const char **syslog_name)
 #ifdef HAVE_SYSLOG_H
   if (!dbgf)
     {
-      static struct log_config lc_syslog;
-      lc_syslog = (struct log_config){
-	.mask = ~0
-      };
-
+      static struct log_config lc_syslog = { .mask = ~0 };
       add_tail(&log_list, &lc_syslog.n);
       *syslog_name = bird_name;
     }
@@ -346,24 +349,15 @@ default_log_list(int initial, const char **syslog_name)
 
   if (dbgf && (dbgf != stderr))
     {
-      static struct log_config lc_debug;
-      lc_debug = (struct log_config){
-	.mask = ~0,
-	.fh = dbgf
-      };
-
+      static struct log_config lc_debug = { .mask = ~0 };
+      lc_debug.fh = dbgf;
       add_tail(&log_list, &lc_debug.n);
     }
 
   if (initial || (dbgf == stderr))
     {
-      static struct log_config lc_stderr;
-      lc_stderr = (struct log_config){
-	.mask = ~0,
-	.terminal_flag = 1,
-	.fh = stderr
-      };
-
+      static struct log_config lc_stderr = { .mask = ~0, .terminal_flag = 1};
+      lc_stderr.fh = stderr;
       add_tail(&log_list, &lc_stderr.n);
     }
 
@@ -371,16 +365,16 @@ default_log_list(int initial, const char **syslog_name)
 }
 
 void
-log_switch(int initial, list *logs, const char *new_syslog_name)
+log_switch(int initial, list *logs, char *new_syslog_name)
 {
   struct log_config *l;
 
-  /* We should not manipulate with log list when other threads may use it */
-  log_lock();
-
   if (!logs || EMPTY_LIST(*logs))
     logs = default_log_list(initial, &new_syslog_name);
 
+  /* We shouldn't close the logs when other threads may use them */
+  log_lock();
+
   /* Close the logs to avoid pinning them on disk when deleted */
   if (current_log_list)
     WALK_LIST(l, *current_log_list)
