@@ -83,6 +83,9 @@ drop_gid(gid_t gid)
 {
   if (setgid(gid) < 0)
     die("setgid: %m");
+
+  if (setgroups(0, NULL) < 0)
+    die("setgroups: %m");
 }
 
 /*
@@ -530,7 +533,7 @@ write_pid_file(void)
 
   /* We don't use PID file for uniqueness, so no need for locking */
 
-  pl = bsnprintf(ps, sizeof(ps), "%ld\n", (long) getpid());
+  pl = bsnprintf(ps, sizeof(ps), "%ld\n", (s64) getpid());
   if (pl < 0)
     bug("PID buffer too small");
 
@@ -598,9 +601,9 @@ cmd_graceful_restart(void)
  *	Signals
  */
 
-volatile int async_config_flag;
-volatile int async_dump_flag;
-volatile int async_shutdown_flag;
+volatile sig_atomic_t async_config_flag;
+volatile sig_atomic_t async_dump_flag;
+volatile sig_atomic_t async_shutdown_flag;
 
 static void
 handle_sighup(int sig UNUSED)
